/*****************************************************************************/
/*****************************************************************************/
/**                       GRAFX95.C  Version 1.02                           **/
/**                   Fonctions graphiques pour le HP95                     **/
/**                                                                         **/
/**   Cration : 25/02/93                          Auteur : Jacques Belin   **/
/**   Dernire modification : Correction bug retour en mode texte (HP100)   **/
/**                                                                         **/
/**  Note : Dvelopp et test uniquement sur Turbo C++ 1.01                **/
/**         TASM ncessaire (pour l'assemblage en ligne de grafx95.c)       **/
/*****************************************************************************/
/*****************************************************************************/
#include "grafx95.h"

#pragma inline                         /* autorisation assemblage en ligne */


/*****************************************************************************/
/* INIT_GRAPH : Positionnement du HP95 en mode graphique                     */
/* Entres : Aucune                                                          */
/* Valeur retourne : Aucune                                                 */
/*****************************************************************************/
void init_graph(void)
   {
    asm   mov   ax,00h
    asm   mov   al,0A0h
    asm   int   5fh
   }


/*****************************************************************************/
/* EXIT_GRAPH : Retour en mode texte                                         */
/* Entres : Aucune                                                          */
/* Valeur retourne : Aucune                                                 */
/*****************************************************************************/
void exit_graph(void)
   {
    asm   mov   ah,00h
    asm   mov   al,87h
    asm   int   5fh
   }


/*****************************************************************************/
/* GET_GRAPH_INFO : Lecture de la table d'entre vido                       */
/* Entres : graphinfo = pointeur sur la table d'entre                      */
/* Sortie  : graphinfo initialis                                            */
/* Valeur retourne : Aucune                                                 */
/*****************************************************************************/
void get_graph_info(GRAPHINFO95 *graphinfo)
   {
    int seg, off;

    seg = FP_SEG((void far *) graphinfo);
    off = FP_OFF((void far *) graphinfo);

    asm   mov   ah,02h
    asm   mov   es,seg
    asm   mov   di,off
    asm   int   5fh
   }


/*****************************************************************************/
/* SET_LOGORIGIN : positionnement de l'origine logique                       */
/* Entres : x, y = coordones de la nouvelle origine logique                */
/* Valeur retourne : Aucune                                                 */
/*****************************************************************************/
void set_logorigin(int x, int y)
   {
    asm   mov   ah,03h
    asm   mov   cx,x
    asm   mov   dx,y
    asm   int   5fh
   }


/*****************************************************************************/
/* SET_CLIP_REGION : Positionnement de la fentre de travail                 */
/* Entres : left, top = coordones du coin suprieur gauche                 */
/*           right, bottom = coordones du coin infrieur droit              */
/* Valeur retourne : Aucune                                                 */
/*****************************************************************************/
void set_clip_region(int left, int top, int right, int bottom)
   {
    asm   mov   ah,04h
    asm   mov   cx,left
    asm   mov   dx,top
    asm   mov   si,right
    asm   mov   di,bottom
    asm   int   5fh
   }


/*****************************************************************************/
/* MOVE_PEN : Positionne le curseur graphique                                */
/* Entres : x, y = coordones du point                                      */
/* Valeur retourne : Aucune                                                 */
/*****************************************************************************/
void move_pen(int x, int y)
   {
    asm   mov   ah,08h
    asm   mov   cx,x
    asm   mov   dx,y
    asm   int   5fh
   }


/*****************************************************************************/
/* SET_COLOR : slection de la couleur courante                              */
/* Entres : color = nouvelle couleur (1:Noir, 0:Blanc)                      */
/* Valeur retourne : Aucune                                                 */
/*****************************************************************************/
void set_color(char color)
   {
    asm   mov   ah,09h
    asm   mov   al,color
    asm   int   5fh
   }


/*****************************************************************************/
/* SET_RULE : Slection de la loi de remplacement                            */
/* Entres : rule = loi de remplacement (FORCE, AND, OR, XOR)                */
/* Valeur retourne : Aucune                                                 */
/*****************************************************************************/
void set_rule(char rule)
   {
    asm   mov   ah,0ah
    asm   mov   al,rule
    asm   int   5fh
   }


/*****************************************************************************/
/* SET_LINE_TYPE : slection de l'aspect des lignes                          */
/* Entres : type = aspect de la ligne (sur 16 bits)                         */
/* Valeur retourne : Aucune                                                 */
/*****************************************************************************/
void set_line_type(int type)
   {
    asm   mov   ah,0bh
    asm   mov   cx,type
    asm   int   5fh
   }


/*****************************************************************************/
/* SET_FILL_MASK : Slection du masque de remplissage                        */
/* Entres : mask = pointeur sur le masque de remplissage (8 octets)         */
/* Valeur retourne : Aucune                                                 */
/*****************************************************************************/
void set_fill_mask(char *mask)
   {
    int seg, off;

    seg = FP_SEG(mask);
    off = FP_OFF(mask);

    asm   mov   ah,01h
    asm   mov   es,seg
    asm   mov   di,off
    asm   int   5fh
   }


/*****************************************************************************/
/* PLOT : Affiche un pixel                                                   */
/* Entres : x, y =coordones du point                                       */
/* Utilise : couleur courante                                                */
/* Valeur retourne : Aucune                                                 */
/*****************************************************************************/
void plot(int x, int y)
   {
    asm   mov   ah,07h
    asm   mov   cx,x
    asm   mov   dx,y
    asm   int   5fh
   }


/*****************************************************************************/
/* GET_PIXEL : Lecture de la couleur d'un pixel                              */
/* Entres : x, y =coordones du point                                       */
/* Valeur retourne : couleur = 1-> Noir, 0->Blanc                           */
/*****************************************************************************/
int get_pixel(int x, int y)
   {
    asm   mov   ah,0ch
    asm   mov   cx,x
    asm   mov   dx,y
    asm   int   5fh
    return(_AX);
   }


/*****************************************************************************/
/* DRAW_LINE : trac d'une ligne  partir du point courant                   */
/* Entres : x_dest y_dest = coordones du point terminal                    */
/* Utilise : couleur, type de ligne et loi de remplacement courantes         */
/* Valeur retourne : Aucune                                                 */
/*****************************************************************************/
void draw_line(int x_dest, int y_dest)
   {
    asm   mov   ah,06h
    asm   mov   cx,x_dest
    asm   mov   dx,y_dest
    asm   int   5fh
   }


/*****************************************************************************/
/* DRAW_RECTANGLE : trac d'un rectangle  partir de la position courante    */
/*                  du curseur, tenant compte de la couleur et de la         */
/*                  loi de remplacement courante.                            */
/* Entres : x_dest, y_dest = coordonnes du coin de fin                     */
/*           fill_type = Type de trac                                       */
/*                        OUTLINE = trac d'un cadre, utilisant le type de   */
/*                                  ligne courant.                           */
/*                        SOLID = Trac d'un rectangle plein, utilisant      */
/*                                la couleur courante                        */
/*                        PATERN = Trac d'un hachurage, tenant compte       */
/*                                 du masque courant                         */
/* Utilise : couleur, type de ligne, masque de remplissage et loi de         */
/*           remplacement courantes                                          */
/* Valeur retourne : Aucune                                                 */
/*****************************************************************************/
void draw_rectangle(int x_dest, int y_dest, char fill_type)
   {
    asm   mov   ah,05h
    asm   mov   cx,x_dest
    asm   mov   dx,y_dest
    asm   mov   al,fill_type
    asm   int   5fh
   }


/*****************************************************************************/
/* GET_IMAGE : Lecture d'une zone graphique dans un buffer                   */
/* Entres : image = pointeur sur un buffer image                            */
/*           x, y = coordonnes du coin suprieur gauche de l'image          */
/*           nb_cols = nombre de colonnes de l'image                         */
/*           nb_lines = nombre de lignes  de l'image                         */
/* Sortie : buffer image initialis                                          */
/* Valeur retourne : Aucune                                                 */
/* NOTE IMPORTANTE : Turbo C utilisant le registre BP pour la pile des       */
/*                   paramtres, il est ncessaire de le sauvegarder avant   */
/*                   de l'utliser pour 'x1'. L'interruption 5F semblant      */
/*                   restituer l'intgralit des registres aprs excution,  */
/*                   j'ai utilis BX pour la sauvegarde. Prions pour         */
/*                   que HP ne change rien dans le futur !                   */
/*****************************************************************************/
void get_image(IMAGE *image, int x, int y, int nb_cols, int nb_lines)
   {
    int seg, off;
    int x1, y1;

    x1 = x+nb_cols-1;
    y1 = y+nb_lines-1;
    seg = FP_SEG((void far *) image);
    off = FP_OFF((void far *) image);

    asm   mov   ah,0dh
    asm   mov   cx,x
    asm   mov   dx,y
    asm   mov   es,seg
    asm   mov   di,off
    asm   mov   si,x1
    asm   mov   bx,bp                            /* sauvegarde de BP dans BX */
    asm   mov   bp,y1
    asm   int   5fh
    asm   mov   bp,bx                                   /* restitution de BP */
   }


/*****************************************************************************/
/* PUT_IMAGE : Affichage d'une image                                         */
/* Entres : image = pointeur sur un buffer image (dj initialis)          */
/*           x, y = coordonnes du coin suprieur gauche de l'image          */
/*           invert = si 1, l'image est inverse avant d'appliquer la loi de */
/*                    remplacement                                           */
/*           rule = loi de remplacement (FORCE, AND, OR, XOR)                */
/* Valeur retourne : Aucune                                                 */
/*****************************************************************************/
void put_image(IMAGE *image, int x, int y, int invert, int rule)
   {
    int seg, off;
    int c;

    c = (invert<<2)+rule;

    seg = FP_SEG((void far *) image);
    off = FP_OFF((void far *) image);

    asm   mov   ah,0eh
    asm   mov   al,c
    asm   mov   cx,x
    asm   mov   dx,y
    asm   mov   es,seg
    asm   mov   di,off
    asm   int   5fh
   }


/*****************************************************************************/
/* WRITE_TEXT : Affichage d'une ligne de texte                               */
/* Entres : x, y = coordonnes du coin suprieur gauche du 1er caractre    */
/*           texte = texte  afficher                                        */
/*           rotate_flag = si diffrent de 0, affichage en vertical          */
/* Valeur retourne : Aucune                                                 */
/*****************************************************************************/
void write_text(int x, int y, char *text, int rotate_flag)
   {
    int seg, off;

    seg = FP_SEG((void far *) text);
    off = FP_OFF((void far *) text);

    asm   mov   ah,0fh
    asm   mov   al,rotate_flag;
    asm   mov   cx,x
    asm   mov   dx,y
    asm   mov   es,seg
    asm   mov   di,off
    asm   int   5fh
   }
