/*****************************************************************************/
/*****************************************************************************/
/**                             GRAFX95.H                                   **/
/**               Fonctions graphiques pour le HP95 (fichier header)        **/
/**                                                                         **/
/**   Cration : 25/02/93                          Auteur : Jacques Belin   **/
/**   Dernire modification : Ajout des motifs de remplissage               **/
/**                                                                         **/
/**  Note : Dvelopp et test uniquement sur Turbo C++ 1.01                **/
/**         TASM ncessaire (pour l'assemblage en ligne de grafx95.c)       **/
/*****************************************************************************/
/*****************************************************************************/

#ifndef  GRAFX95
#define  GRAFX95 1

#include <dos.h>

#define  CWHITE    0                              /* dfinition des couleurs */
#define  CBLACK    1

                                   /* Codes de fonctions pour draw_rectangle */
#define  OUTLINE  0                                        /* trac de cadre */
#define  SOLID    1                                       /* rectangle plein */
#define  PATERN   2                                             /* hachurage */

                                                     /* Lois de remplacement */
#define  FORCE    0
#define  AND      1
#define  OR       2
#define  XOR      3

#define  NORMAL   0                          /* flag d'inversion d'une image */
#define  INVERSE  1

#define  SOLID_LINE   0xFFFF                     /* types de lignes standard */
#define  DOTTED_LINE  0xAAAA
#define  CENTER_LINE  0xEEEE
#define  DASHED_LINE  0xFAFA

                              /* motifs de remplissages dfinis dans turbo C */
#define  EMPTY_FILL       "\x00\x00\x00\x00\x00\x00\x00\x00"
#define  SOLID_FILL       "\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF"
#define  LINE_FILL        "\xFF\xFF\x00\x00\xFF\xFF\x00\x00"
#define  LTSLASH_FILL     "\x01\x02\x04\x08\x10\x20\x40\x80"
#define  SLASH_FILL       "\xE0\xC1\x83\x07\x0E\x1C\x38\x70"
#define  BCKSLASH_FILL    "\xF0\x78\x3C\x1E\x0F\x87\xC3\xE1"
#define  LTBKSLASH_FILL   "\xA5\xD2\x69\xB4\x5A\x2D\x96\x4B"
#define  HATCH_FILL       "\xFF\x88\x88\x88\xFF\x88\x88\x88"
#define  XHATCH_FILL      "\x81\x42\x24\x18\x18\x24\x42\x81"
#define  INTERLEAVE_FILL  "\xCC\x33\xCC\x33\xCC\x33\xCC\x33"
#define  WIDE_DOT_FILL    "\x80\x00\x08\x00\x80\x00\x08\x00"
#define  CLOSE_DOT_FILL   "\x88\x00\x22\x00\x88\x00\x22\x00"

                                                           /* Autres motifs */
#define  GRAY_FILL        "\xAA\x55\xAA\x55\xAA\x55\xAA\x55"
#define  ZIGZAG_FILL      "\x88\x41\x22\x14\x88\x41\x22\x14"
#define  CROSS_FILL       "\x40\xE0\x40\x00\x04\x0E\x04\x00"
#define  ALU_FILL         "\x09\x12\x24\x48\x90\x21\x42\x84"
#define  BRASS_FILL       "\x11\x22\x44\x88\x10\x20\x40\x80"
#define  PLASTIC_FILL     "\xFF\x40\x20\x10\x08\x04\x02\x01"
#define  BRIKS1_FILL      "\xFF\x80\x80\x80\xFF\x08\x08\x08"
#define  BRIKS2_FILL      "\xFC\x84\xFC\x00\xCF\x48\xCF\x00"
#define  BRIKS3_FILL      "\x01\x02\x06\x09\x90\x60\x40\x80"
#define  BRIKS4_FILL      "\x44\x0A\x11\x22\x44\x88\x11\xA0"
#define  TILE1_FILL       "\x80\x41\x22\x1C\x08\x14\x22\xC1"
#define  TILE2_FILL       "\x82\x82\x44\x44\x2A\x2A\x11\x11"
#define  GRASS_FILL       "\x20\xA8\x70\x20\x02\x8A\x07\x02"
#define  SPIRAL_FILL      "\xFE\x80\xBE\xA2\xBA\x82\xFE\x00"
#define  DIAMS_FILL       "\x22\x50\x88\x50\x22\x05\x88\x05"
#define  FACES_FILL       "\x7C\x82\xAA\x82\xBA\x82\x7C\x00"
#define  CLOTH_FILL       "\xB8\x11\x8B\xC7\xA3\x11\x3A\x7C"
#define  CUBES_FILL       "\x3E\x46\xFE\x8E\xAE\x8C\xF8\x00"
#define  SCALES_FILL      "\xC1\x32\x0C\x04\x02\x02\x01\x01"
#define  PUZZLE_FILL      "\x01\x03\x04\x04\x23\x51\x51\xDF"



typedef struct                             /* Table d'informations graphique */
         {
          char vmode;                                  /* mode vido courant */
          char def_vmode;                                 /* mode par dfaut */
          int  disp_width;                   /* largeur de l'cran en pixels */
          int  disp_eight;                   /* hauteur de l'cran en pixels */
          int  pen_col;             /* colonne courante du curseur graphique */
          int  pen_row;               /* ligne courante du curseur graphique */
          int  line_type;                           /* type de ligne courant */
          int  rep_rule;                     /* loi de remplacement courante */
          int  color;                                    /* couleur courante */
          int  left_clip;           /* limite gauche de la fentre graphique */
          int  top_clip;             /* limite haute de la fentre graphique */
          int  right_clip;          /* limite droite de la fentre graphique */
          int  bottom_clip;           /* limite base de la fentre graphique */
          int  col_orig;                     /* colonne de l'origine logique */
          int  row_orig;                       /* ligne de l'origine logique */
          char fill_mask[8];                 /* motif de remplissage courant */
         } GRAPHINFO95;


typedef struct                                               /* buffer image */
        {
         int  planes;                        /* nombre de plans (1 sur HP95) */
         int  bits_by_pixel;        /* nombre de bits par pixel (1 sur HP95) */
         int  width;                          /* nombre de pixels en largeur */
         int  height;                         /* nombre de pixels en hauteur */
         char image[2];                         /* premier octet de l'image */
        } IMAGE;


                /* Macro permettant de calculer le la taille du buffer image */
#define LEN_IMAGE(width, height)    (8+(((width+7)/8)*height))

/*****************************************************************************/
/* DESCRIPT_IMAGE : descripteur d'un buffer Image                            */
/* paramtres : wdth, hght = largeur, hauteur de l'image en pixels           */
/*              NAME = type de structure                                     */
/* ATTENTION : Cette macro ne dcrit que le type de structure (typedef).     */
/*             Elle doit tre suivie d'une dclaration de variable utilisant */
/*             ce type de structure.                                         */
/*****************************************************************************/
#define DESCRIPT_IMAGE(wdth, hght, NAME)  typedef struct \
        {                                                \
         int  planes;                                    \
         int  bits_by_pixel;                             \
         int  width;                                     \
         int  height;                                    \
         char image[(((wdth)+7)/8)*(hght)];              \
        } NAME;




                         /* prototypes des fonctions incluses dans grafx95.c */
void init_graph(void);
void exit_graph(void);
void get_graph_info(GRAPHINFO95 *graphinfo);
void set_logorigin(int x, int y);
void set_clip_region(int left, int top, int right, int bottom);
void move_pen(int x, int y);
void set_color(char color);
void set_rule(char rule);
void set_line_type(int type);
void set_fill_mask(char *mask);
void plot(int x, int y);
int  get_pixel(int x, int y);
void draw_line(int x_dest, int y_dest);
void draw_rectangle(int x_dest, int y_dest, char fill_type);
void get_image(IMAGE *image, int x, int y, int nb_cols, int nb_lines);
void put_image(IMAGE *image, int x, int y, int invert, int rule);
void write_text(int x, int y, char *text, int rotate_flag);

#endif
