/*****************************************************************************/
/*****************************************************************************/
/**                               MAKEA95.C                                 **/
/**                 Construction des fichiers anim HP95                     **/
/**                                                                         **/
/**   Dernire modification : 27/05/94              Auteur : Jacques Belin  **/
/**      ajout de l'option "s"                                              **/
/**                                                                         **/
/*****************************************************************************/
/*****************************************************************************/
#include <stdio.h>
#include <dir.h>
#include <dos.h>
#include <fcntl.h>
#include <string.h>

#include "..\getopt.c"        /* Routine d'analyse des options (voir JPC 78) */

#define MAGIC    0x5435           /* code de reconnaissance des fichiers A95 */

#define NOCOMP   0                         /* modes de compression supports */
#define PCXCOMP  1

#define ONETIME  0                                      /* types d'animation */
#define LOOP     1
#define PINGPONG 2


typedef struct                                    /* Entte des fichiers PCX */
       {
        char     manuf;                  /* numro fabricant (toujours 0x0A) */
        char     vers;                                  /* numro de version */
        char     rle;                                /* flag compilation RLE */
        char     bitpx;                          /* nombre de bits par pixel */
        unsigned left;
        unsigned top;
        unsigned right;
        unsigned bottom;
        unsigned hres;
        unsigned vres;
        unsigned char rgb[48];
        char     vmode;
        char     nplanes;
        unsigned bplin;
        unsigned palinfo;
        unsigned shres;
        unsigned svres;
        char     filer[54];
       } PCX_HEADER;

typedef struct                                    /* entte des fichiers A95 */
       {
        unsigned      magic;                       /* code de reconnaissance */
        char          vers;                            /* version du fichier */
        char          comp;                           /* mode de compression */
        unsigned      nimages;                 /* nombre d'images du fichier */
        char          anim;                              /* type d'animation */
        unsigned char speed;                               /* vitesse idale */
        char          filer[120];             /* rserv pour un usage futur */
       } A95_HEADER;

int fd_in;                                           /* handles des fichiers */
int fd_out;

char far *ncbuff;                      /* pointeurs sur les buffers utiliss */
char far *cbuff;


cdecl main(int argc, char *argv[]);

main(int argc, char *argv[])
    {
     int errflag=0;                                         /* flag d'erreur */
     struct ffblk dirdesc;               /* structure d'entre de rpertoire */
     int findret;                    /* flac de fin de recherche de fichiers */
     char destname[80];                     /* stockage des noms de fichiers */
     char filename[80];
     char *path;                              /* pointeur sur nom de fichier */
     int nbfiles=0;                                /* nombre de fichiers lus */
     int nbytes;                            /* taille d'un fichier graphique */
     unsigned ncseg, cseg;               /* numros des segments des buffers */
     int ioflag;                       /* resultat des oprations de lecture */
     int n, i, j, k;
     char c;

     PCX_HEADER pcxhdr;
     A95_HEADER a95hdr;

     char typeanim=LOOP;                               /* valeurs par dfaut */
     char typecomp=PCXCOMP;
     unsigned char speed=24;

     while((c=getopt(argc, argv, "a:c:s:")) != EOF)
       {
        switch(c)                                   /* selection des options */
          {
           case 'a' : typeanim=atoi(optarg);             /* type d'animation */
                      if(typeanim > PINGPONG)
                         errflag++;
                      break;

           case 'c' : typecomp=atoi(optarg);          /* mode de compression */
                      if(typecomp > PCXCOMP)
                         errflag++;
                      break;

           case 's' : speed=atoi(optarg);                         /* vitesse */
                      if((speed < 1) || (speed > 255))
                         errflag++;
                      break;

           case '?' : errflag++;
                      break;
          }
       }


     if((argc-optind) < 2)                     /* si pas assez de paramtres */
        errflag++;

     if(errflag)                                   /* sortie en cas d'erreur */
       {
        fprintf(stderr, "usage : makea95 [-a type ] [-c mode] [-s speed] "
                                                 "anim_file files [files...]\n"
                        "\n"
                        "           type  = animation type :\n"
                        "                    0 = one time\n"
                        "                    1 = loop\n"
                        "                    2 = ping-pong\n"
                        "           mode  = compression mode :\n"
                        "                    0 = no compression\n"
                        "                    1 = PCX\n"
                        "           speed = ideal display speed\n"
                        "                    between 1 and 255\n"
                        "           anim_file = A95 file\n"
                        "           files [files...] = 240*128 mono PCX files\n"
                        "\n"
                        "(c) Jacques Belin, 1994\n" );
        exit(-1);
       }

     strcpy(destname, argv[optind]);
     optind++;

     allocmem(8000/16, &ncseg);                    /* allocation des buffers */
     allocmem(8000/16, &cseg);
     ncbuff = MK_FP(ncseg, 0);
     cbuff  = MK_FP(cseg, 0);


     memset(&a95hdr, 0, 128);                  /* initialisation de l'entte */
     a95hdr.magic = MAGIC;
     a95hdr.vers  = 1;
     a95hdr.comp  = typecomp;
     a95hdr.anim  = typeanim;
     a95hdr.speed = speed;


     fd_out = _creat(destname, FA_ARCH);          /* cration du fichier A95 */
     if(fd_out == -1)                                           /* si erreur */
       {
        fprintf(stdout, "Unable to create file %s\n", destname);
        errflag=1;
       }
     else
       {
        if(write(fd_out, &a95hdr, 128) != 128)         /* criture du buffer */
          {
           fprintf(stdout, "Unable to write file %s\n", argv[optind]);
           errflag=1;
          }

        for(; (optind<argc) && !errflag; optind++)  /* analyse des oprandes */
          {
           path = filename + strlen(argv[optind])-1;    /* analyse du chemin */
           strcpy(filename, argv[optind]);
           while((path >= filename) && (*path != '\\') && (*path != ':'))
              path--;
           path++;
           findret = findfirst(argv[optind], &dirdesc, 0); /* recherche noms */
           if(findret != 0)                      /* si pas de fichier trouv */
             {
              fprintf(stdout, "%s  not found\n", argv[optind]);
              errflag++;
             }
           while(!findret)              /* tant que l'on trouve des fichiers */
             {
              strcpy(path, dirdesc.ff_name);        /* constitution path+nom */
              fd_in=open(filename, O_RDONLY | O_BINARY);
              if(fd_in == NULL)                     /* si erreur d'ouverture */
                {
                 fprintf(stdout, "unable to open  %s\n", filename);
                 errflag++;
                }
              else
                {
                 if(read(fd_in, &pcxhdr, 128) != 128)
                   {
                    fprintf(stdout, "unable to read  %s\n", filename);
                    errflag++;
                   }

                 if(!errflag)        /* test (sommaire) validit fichier PCX */
                   {
                    if((pcxhdr.manuf  != 10) || (pcxhdr.rle != 1) ||
                       (pcxhdr.bitpx  != 1)  || (pcxhdr.right != 239) ||
                       (pcxhdr.bottom != 127) )
                      {
                        fprintf(stdout, "unable to read  %s\n", filename);
                        errflag++;
                      }
                   }

                 if(!errflag)
                   {
                    nbfiles++;
                    fprintf(stdout, "%s\n", filename);      /* affichege nom */

                    nbytes = read(fd_in, cbuff, 9999);

                    fprintf(stdout, "   %5d bytes read\n", nbytes);

                    switch(typecomp)
                      {
                       case NOCOMP  : k = 0;    /* dcompression fichier PCX */
                                      for(i=0; i<nbytes; i++)
                                        {
                                         c = cbuff[i];
                                         if (((c >> 6) & 3) == 3)  /* si rle */
                                           {
                                            n = c & 0x3f;
                                            c=cbuff[++i];
                                            for(j=1; j<=n; j++)
                                               ncbuff[k++]=c;
                                           }
                                         else
                                           {
                                            ncbuff[k++]=c;
                                           }
                                        }

                                      nbytes = 3840;
                                      if(write(fd_out, &nbytes, 2) != 2)
                                         errflag++;
                                      if(write(fd_out, ncbuff, nbytes)
                                           != nbytes)
                                         errflag++;
                                      if(write(fd_out, &nbytes, 2) != 2)
                                         errflag++;
                                      break;

                       case PCXCOMP : if(write(fd_out, &nbytes, 2) != 2)
                                         errflag++;
                                      if(write(fd_out, cbuff, nbytes)
                                           != nbytes)
                                         errflag++;
                                      if(write(fd_out, &nbytes, 2) != 2)
                                         errflag++;
                                      break;
                      }

                    if(errflag)                      /* si erreur d'ecriture */
                      {
                       fprintf(stdout, "unable tu write  %s\n", destname);
                      }
                    else
                      {
                       fprintf(stdout, "   %5d bytes written\n", nbytes);
                      }
                    close(fd_in);                   /* fermeture fichier PCX */
                   }
                }
              findret = findnext(&dirdesc);
             }
          }
       }

     if(!errflag)                                         /* si pas d'erreur */
       {
        a95hdr.nimages = nbfiles;    /* criture nombre d'images dans fichier*/
        lseek(fd_out, 0L, SEEK_SET);
        write(fd_out, &a95hdr, 128);
                                               /* affichage du rapport final */
        fprintf(stdout, "\nCopied in %s\n\n", destname);

        fprintf(stdout, "Number of pict.  = %d\n", a95hdr.nimages);
        fprintf(stdout, "Animation type   = ");
        switch(typeanim)
          {
           case ONETIME  : fprintf(stdout, "One time\n");
                           break;

           case LOOP     : fprintf(stdout, "Loop\n");
                           break;

           case PINGPONG : fprintf(stdout, "Ping Pong\n");
                           break;

           default       : fprintf(stdout, "Unknown\n");
                           break;
          }

        fprintf(stdout, "Compression mode = ");
        switch(typecomp)
          {
           case NOCOMP   : fprintf(stdout, "No compression\n");
                           break;

           case PCXCOMP  : fprintf(stdout, "PCX\n");
                           break;

           default       : fprintf(stdout, "Unknown\n");
                           break;
          }

        fprintf(stdout, "Speed            = %d\n\n", speed);
       }

     close(fd_out);                                  /* fermeture du fichier */

     freemem(ncseg);                               /* libration des buffers */
     freemem(cseg);
    }

