/*****************************************************************************/
/*****************************************************************************/
/**                                 ACP.C                                   **/
/**    Conversion des fichiers gnrs par AREUH en LEX sur disquette LIF   **/
/**                                                                         **/
/**    Cration : 30/10/90               Auteur : JB (bas sur COPY.H       **/
/**                                                      de pda et JT)      **/
/**    Derniere modification : 16/02/91                                     **/
/**       Ajout du paramtre LIF_NAME                                       **/
/**                                                                         **/
/*****************************************************************************/
/*****************************************************************************/
#include "lif.c"

#if defined(__TURBOC__)
#include "lif.tc"
#else
#include "lif.msc"
#endif

long read_nib(FILE *fp, int c);         /* prototype des fonctions utilises */
void skip(FILE *fp, int n);
int  hex(unsigned char c);

void main(int argc, char *argv[])
   {
    int    lif_drive=0;         /* numro du lecteur de disquette (0=A, 1=B) */
    int    opt_file=0;   /* flag de prsence de nom de fichier LIF optionnel */
    int    err_entry=0;                /* flag d'erreur entre de parametres */
    FILE   *fp;                             /* descripteur du fichier MS/DOS */
    LIFDIR dir;                 /* structure contenant l'entre du directory */
    int    pos_dir;                       /* numro de l'entre du directory */
    char   lif_name[11], dos_name[50];                   /* nom des fichiers */
    GDH    gdh;                    /* structure contenant la date du fichier */
    int    long_file;          /* longueur du fichier LIF en enregistrements */
    int    first_sect;              /* premier enregistrement du fichier LIF */
    int    cl_dir;             /* si dplacement de fichier, ancienne entre */
    long   nb_bytes;                              /* nombres d'octets du LEX */
    unsigned long magic;                      /* code de validit du fichier */
    unsigned long lnib;                       /* longueur fichier en nibbles */
    int    i;

    if((argc<2) || (argc>3))               /* 1 ou 2 paramtres obligatoires */
      {
       err_entry=1;
      }
    else
      {
       if(argc==3)                                        /* si 2 paramtres */
         {
                  /* la longueur doit tre comprise entre 2 et 12 caractres */
          if((strlen(argv[2]) < 2) || (strlen(argv[2]) > 12))  
             err_entry = 1;
          lif_drive=(int) argv[2][0]-'0';      /* controle numro de lecteur */
          if((lif_drive < 0) || (lif_drive > 1))
             err_entry=1;
          if(!err_entry)
            {
             if(argv[2][1] != ':')            /* controle deuxime caractre */
                err_entry=1;
             if(!err_entry && (strlen(argv[2]) > 2))  /* acquisition nom LIF */
               {
                opt_file=1; /* armement du flag de prsence de nom optionnel */
                strcpy(lif_name, argv[2]+2);
                                            /* ajout d'espaces en fin du nom */
                for(i=strlen(lif_name); i<10; i++)     
                   lif_name[i]=' ';
                lif_name[10]='\0';
               }
            }
         }
      }

    if(err_entry)                                               /* si erreur */
      {
       fprintf(stderr, "usage : acp file [drive_number:[LIF_NAME]]\n");
       exit(-1);
      }

    strcpy(dos_name, argv[1]);            /* rcupration nom du fichier DOS */

    init_dsk();                                 /* initialisation du lecteur */

    fp=fopen(dos_name, "rb");                    /* ouverture du fichier DOS */
    if(fp == NULL)                                        /* si non trouv...*/
      {
       fprintf(stdout, "file %s not found\n", dos_name);
       fin(-1);
      }

                              /* lecture de la valeur de controle du fichier */
    fread ((char *) &magic, sizeof (unsigned long), 1 , fp);
    if (magic!=0x1b080100L)                         /* si code different ... */
      {
       fprintf(stdout, "Error : %s is not a LEX file\n", dos_name);
       fin(-1);
      }

    if(opt_file)               /* si prsence du nom de fichier en paramtre */
      {
       skip(fp, 32);         /* on saute le nom, le type, la date et l'heure */
      }
    else
      {
       for(i=0; i<8; i++)              /* lecture et formatage du nom du LEX */
          lif_name[i] = (unsigned char) read_nib(fp, 2);
       lif_name[8] = lif_name[9] = ' ';
       lif_name[10] = '\0';
       skip(fp, 16);      /* on saute le type, la date et l'heure du fichier */
      }

                         /* lecture de la longueur en nibbles du fichier LEX */
    lnib = ((unsigned long) read_nib(fp, 5)) - (unsigned long) 5;
                                 /* calcul de la longueur en enregistrements */
    long_file = (int) (lnib / 512) + ((lnib % 512) ? 1 : 0) ;


                   /* recherche d'une place sur la disquette pour la fichier */
    first_sect=best_room(lif_drive, lif_name, long_file, &pos_dir, &cl_dir);
    if(first_sect == -3)
      {
       fprintf(stdout, "Error : Not a LIF disk\n");
       fin(-1);
      }
    if(first_sect == -1)
      {
       fprintf(stdout, "Not enough space for this file\n");
       fin(-1);
      }

    curr_sect=first_sect; /* initialisations des pointeurs pour 'write char' */
    curr_char=0;
    nb_bytes = ((lnib % 2) ? lnib+1 : lnib) >> 1;  /* longueur LEX en octets */
    while(nb_bytes--)
      {                                  /* lecture et conversion du fichier */
       write_char(lif_drive, (unsigned char) read_nib(fp, 2));
      }
    memset(buff_sect+curr_char, 0x00, 256-curr_char); /*normalisation fin enr*/
    wrt_dsk(lif_drive, curr_sect, (char *) buff_sect); /*criture dernier enr*/

    fclose(fp);                                  /* fermeture du fichier DOS */

    get_dos_gdh(dos_name, &gdh);     /* lecture date et heure du fichier DOS */

                                         /* construction entre du directory */
    build_dir(&dir, lif_name, 0, 0, LEX, first_sect, long_file, &gdh,lnib, 0L);
    wrt_dir(lif_drive, pos_dir, &dir);       /* ecriture entre du directory */

    if(cl_dir != -1)                                   /* si fichier dplac */
       clr_file(lif_drive, cl_dir);           /* on efface l'ancienne entre */

    fin(0);               /* fin du programme et rinitialisation du lecteur */
   }


/*****************************************************************************/
/*   READ_NIB : lit, inverse et convertit 'n' nibbles du fichier             */
/*   entres : fp = descripteur de fichier DOS                               */
/*             n = nombre de nibbles  lire                                  */
/*   valeur retourne : nibbles lus, convertis en dcimal                    */
/*****************************************************************************/
long read_nib(FILE *fp, int n)
   {
    unsigned char buf[16];              /* buffer d'inversion des caractres */
    int i;
    long res;                                             /* valeur calcule */

    res = 0;
    for (i=0; i<n; i++)
      {                /* lecture de 'n' caractres (nibbles) dans le buffer */
       buf[i] = (unsigned char) getc(fp);
      }
    for(i=n-1; i>=0; i--)                 /* lecture buffer en ordre inverse */
       res = (res << 4) | hex (buf [i]);  /* calcul res = res*16+hex(buf[i]) */
    return (res);
   }

/*****************************************************************************/
/*   SKIP : Saute d'un nombre 'n' de caractres dans le fichier              */
/*   entres : fp = descripteur de fichier DOS                               */
/*             n = nombre de nibbles  sauter                                */
/*   valeur retourne : aucune                                               */
/*****************************************************************************/
void skip(FILE *fp, int n)
   {
    unsigned char buf[32];                            /* buffer de rception */

    fread(buf, n, 1, fp);                          /* lecture de 'n' nibbles */
   }

/*****************************************************************************/
/*   HEX : convertit un caractere hexadecimal en nombre dcimal              */
/*   entre : c = caractre  convertir                                      */
/*   valeur retourne : nombre dcimal                                       */
/*****************************************************************************/
int hex(unsigned char c)
   {
    return (((c>='A')&&(c<='F')) ? c-'A'+10 : c-'0') ;
   }

